C
C
C *******************************************************************
C
C     Planar Earth-Mars Transfer
C
C     Orbit Transfer Problem with Minimum Fuel Control
C     Control Structure: Bang-Bang 
C
C     Here: Solution with two Thrust Subarcs
C
C     H.J. Oberle, K. Taubert: Existence and Multiple Solutions of the
C     Minimum-Fuel Orbit Transfer Problem.
C     Journ. of Optim. Theory and Applic., Vol 95, pp. 243-262, 1997.
C
C
C *******************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (MMAX=11,MSMAX=9,MMS=MMAX+MSMAX,NMS=20,
     *           NDW=NMS*(8+MMAX*(10+6*NMS)),NDIW=5*NMS,
     *           NFILE=6,NPLOT=7)
      DIMENSION X(MMS),XS(MSMAX),Y(NMS,MMS),WORK(NDW)
      INTEGER JS(MMS,MSMAX),IWORK(NDIW)
      EXTERNAL F,R,DIFSYB,RKF7B
      COMMON /ITEIL/ ITEIL
      COMMON /PARA/ BEMAX,C,RF,PI,TF,HAM,SWIT,BETA,PSI
C
      OPEN(6,FILE='mars.txt',STATUS='UNKNOWN')
      OPEN(7,FILE='mars.dat',STATUS='UNKNOWN')
      OPEN(8,FILE='mars.krs',STATUS='UNKNOWN')
C
      N    = 11
      M    = 11
      MS   =  2
      TOL  =  1.D-8
C
      PI   =  3.1415926535898D0
      PI   =  2.D0*PI
      BEMAX=  0.075D0
      C    =  1.872D0
      RF   =  1.525D0
      HAM  =  0.D0
C
C --- Initial trajectory
C
      Y(1,1)  =  1.0D0
      Y(2,1)  =  0.0D0
      Y(3,1)  =  1.0D0
      Y(4,1)  =  1.0D0
      Y(5,1)  = -0.5D0
      Y(6,1)  =  0.0D0
      Y(7,1)  = -0.5D0
      Y(8,1)  = -0.9D0
      Y(9,1)  =  0.5D1
      Y(10,1) =  0.0D0
      Y(11,1) =  0.0D0
C
      XS(1)   =  0.11D0
      XS(2)   =  0.89D0
C  
      DO 100 K=1,M
      X(K)=FLOAT(K-1)/FLOAT(M-1)
      DO 100 I=1,11
      Y(I,K)  =  Y(I,1)
  100 CONTINUE
C
C
C     First Call of BNDSCO 
C     (for Completion of the Initial Data; ITMAX = -1)
C
      ITEIL  =  1
      ITMAX  = -1
      KS     =  1
      KP     =  0
      CALL BNDSCO(F,R,DIFSYB,X,XS,Y,WORK,IWORK,JS,N,M,MS,
     *     KS,TOL,ITMAX,KP,MMAX,MSMAX,MMS,NMS,NDW,NDIW,NFILE)
      IF(KP.LT.0) GOTO 900
      CALL RESET(X,XS,Y,N,M,MS,NMS)
C
C
C     Solution of the Boundary Value Problem
C
      ITMAX = 20
      KS    = 1
      KP    = 0
      CALL BNDSCO(F,R,RKF7B,X,XS,Y,WORK,IWORK,JS,N,M,MS,
     *     KS,TOL,ITMAX,KP,MMAX,MSMAX,MMS,NMS,NDW,NDIW,NFILE)
      IF(KP.LT.0) GOTO 900
C
C     Numerical Integration of the Solution Trajectory,
C     Determination of the Data for Plot with MATLAB
C     Files: mars***.dat, mars***.krs
C 
      ITEIL = 2
      KP    = 0
      IFEIN = 5
      IPLOT = 1
      CALL AWP(DIFSYB,F,X,XS,Y,N,M,MS,
     *               JS,MMS,NMS,IFEIN,IPLOT,KP,NFILE,NPLOT)
C
  900 CONTINUE
C
 1001 FORMAT(4D20.12)
C -------------------------------------------------------------------
      STOP
      END




      SUBROUTINE F(X,Y,DY,J,L,JS,MMS)
C
C     Planar Earth-Mars Transfer
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(*),DY(*)
      INTEGER JS(MMS,*)
      COMMON /ITEIL/ ITEIL
      COMMON /PARA/ BEMAX,C,RF,PI,TF,HAM,SWIT,BETA,PSI
C
      RR   =  1.D0/Y(1)
      W    =  Y(2)
      V    =  Y(3)
      XM   =  Y(4)
      RL   =  Y(5)
      WL   =  Y(6)
      VL   =  Y(7)
      TF   =  Y(9)
      PHI  =  Y(10)
      PHIL =  Y(11)
C
      RL2  =  RL*RL
      WL2  =  WL*WL
      VL2  =  VL*VL
      W1   =  SQRT(WL2+VL2)
      WR   =  W*RR
      VR   =  V*RR
      R2   =  RR*RR
      R3   =  RR*R2
C
      PSI  =  DATAN(WL/VL)
C
      BETA =  BEMAX
      IF(JS(J,1)*JS(J,2).LT.0) BETA=0.D0
C
      A    =  C*BETA/XM
      A1   =  A/W1
      A2   =  WR*VL
      A3   =  PHIL*RR
      SWIT = -C*W1/XM-Y(8)
C
      DY(1)  =  TF*W
      DY(2)  =  TF*(VR*V-R2-A1*WL)
      DY(3)  =  TF*(-W*VR-A1*VL)
      DY(4)  = -TF*BETA
C
      DY(5)  =  TF*(WL*(VR*VR-2.D0*R3) - A2*VR + A3*VR)
      DY(6)  =  TF*(-RL + VL*VR)
      DY(7)  =  TF*(-2.D0*VR*WL + A2 - A3)
      DY(8)  = -TF*A*W1/XM
C
      DY(9)  =  0.D0
C
      DY(10) =  TF*VR
      DY(11) =  0.D0
C -------------------------------------------------------------------
      RETURN
      END




      SUBROUTINE R(YA,YB,ZZ,W,NYA,NSK,J,L,LS,JS,MMS)
C
C     Planar Earth-Mars Transfer
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YA(*),YB(*),ZZ(*),W(*)
      INTEGER NYA(*),NSK(*)
      INTEGER JS(MMS,*)
      COMMON /ITEIL/ ITEIL
      COMMON /PARA/ BEMAX,C,RF,PI,TF,HAM,SWIT,BETA,PSI
C
      SWB = -C/YB(4)*SQRT(YB(6)*YB(6)+YB(7)*YB(7))-YB(8)
C
      W(1)  =  YA(2)
      W(2)  =  YA(1)-1.D0
      W(3)  =  YA(3)-1.D0
      W(4)  =  YA(4)-1.D0
      W(5)  =  YA(10)
      W(6)  =  YB(8)+1.D0
      W(7)  =  YB(1)/RF-1.D0
      W(8)  =  YB(2)
      W(9)  =  YB(3)*YB(3)*RF-1.D0
      W(10) =  SWB
      W(11) =  YA(11)
C
      NYA(1)  =  2
      NYA(2)  =  1
      NYA(3)  =  3
      NYA(4)  =  4
      NYA(10) =  5
C
      W1  =  SQRT(ZZ(6)*ZZ(6)+ZZ(7)*ZZ(7))
      SW  = -C*W1/ZZ(4)-ZZ(8)
C
      W(12) =  SW
      W(13) =  SW  
      NSK(12) =  1
      NSK(13) =  2
C
C -------------------------------------------------------------------
      RETURN
      END




      SUBROUTINE AWP(METHOD,F,X,XS,Y,N,M,MS,
     *               JS,MMS,NMS,IFEIN,IPLOT,KP,NFILE,NPLOT)
C
C *******************************************************************
C
C     INTEGRATION OF THE SOLUTION TRAJECTORY
C
C     IFEIN : NUMBER OF INTEGRATION NODES IN SUBINTERVAL
C     IPLOT : THE NODES ARE STORED FOR PLOT SUBROUTINE,
C             IF IPLOT.EQ.1
C     KP    : PRINT PARAMETER
C             KP.LT.0 : NO PRINT
C             KP.EQ.0 : MULTIPLE SHOOTING NODES
C             KP.GT.0 : INTEGRATION NODES
C
C *******************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(*),XS(*),Y(NMS,*)
      DIMENSION Z(20),DZ(20)
      INTEGER JS(MMS,*)
      EXTERNAL F,R
      COMMON /ITEIL/ ITEIL
      COMMON /PARA/ BEMAX,C,RF,PI,TF,HAM,SWIT,BETA,PSI
C
      IF(IPLOT.GT.0) IFEIN=40
      TOL=1.D-07
C
      M1=M-1
      NN=N
C
      JT=0
      L=0
      HI=0.D0
      IF(KP.GE.0) THEN
        WRITE(NFILE,2000)
        WRITE(NFILE,2200)
      ENDIF
C
      DO 500 K=1,M1
      J=K
      J1=J+1
      L=L+1
C
      DO 501 I=1,N
  501 Z(I)=Y(I,J)
C
      T=X(J)
      TEND=X(J1)
      IF(MS.GT.0) THEN
         DO 503 KL=1,MS
  503    IF(T.EQ.XS(KL)) L=L-1
      ENDIF
      HMAX=(TEND-T)/IFEIN
      IF(HI.EQ.0.D0) HI=0.5D0*HMAX
      HMAX=ABS(HMAX)
C
      CALL F(T,Z,DZ,J,L,JS,MMS)
C
      IF(KP.GE.0) WRITE(NFILE,2001)
      JT=JT+1
      IF(KP.GE.0)
     *WRITE(NFILE,3000) T,(Z(I),I=1,10),SWIT,BETA,PSI
      IF(IPLOT.EQ.1)
     *WRITE(NPLOT,4000) T,(Z(I),I=1,10),SWIT,BETA,PSI
C
C     ABSPEICHERN DER SCHALTPUNKTE
C
      IF(ABS(SWIT).LT.1.D-9) THEN
         XP = Z(1)*DCOS(Z(10))
         YP = Z(1)*DSIN(Z(10))
         WRITE(8,4000) XP,YP
      ENDIF
C
      DO 510 K1=1,IFEIN
      T1=T+SIGN(HMAX,HI)
      IF(K1.EQ.IFEIN) T1=TEND
C
      CALL METHOD(NN,F,T,Z,T1,TOL,HMAX,HI,J,L,JS,MMS)
C
      IF(HI.EQ.0.D0) GOTO 520
C
      CALL F(T,Z,DZ,J,L,JS,MMS)
C
      JT=JT+1
      IF(KP.GT.0)
     *WRITE(NFILE,3000) T,(Z(I),I=1,10),SWIT,BETA,PSI
      IF(IPLOT.EQ.1)
     *WRITE(NPLOT,4000) T,(Z(I),I=1,10),SWIT,BETA,PSI
C
  510 CONTINUE
C
  520 CONTINUE
  500 CONTINUE
C
      IF(KP.EQ.0) THEN
         WRITE(NFILE,2001)
         WRITE(NFILE,3000) T,(Z(I),I=1,10),SWIT,BETA,PSI
      ENDIF
      IF(KP.GE.0) WRITE(NFILE,2001)
C
      WRITE(NFILE,5001) Z(9)
      WRITE(NFILE,5002) Z(4)
      WRITE(NFILE,5003) Z(10)
      PM = Z(10) - Z(9)/(RF*SQRT(RF))
      WRITE(NFILE,5004) PM
C
C     ABSPEICHERN DES ENDPUNKTES
C
      XP = Z(1)*DCOS(Z(10))
      YP = Z(1)*DSIN(Z(10))
      WRITE(8,4000) XP,YP
C
C     ABSPEICHERN DER ANFANGS-MARS-POSITION
C
      XP = RF*DCOS(PM)
      YP = RF*DSIN(PM)
      WRITE(8,4000) XP,YP
C
 1000 FORMAT(I4)
 2000 FORMAT(///)
 2001 FORMAT(' ',78('*'))
 2200 FORMAT(7X,'T/TF',14X,'R',17X,'W',17X,'V'/
     *                 25X,'M',17X,'LR',16X,'LW'/
     *                 25X,'LV',16X,'LM',16X,'TF'/
     *                 25X,'PHI',15X,'S',17X,'BETA'/
     *                 25X,'PSI')
 3000 FORMAT(4D18.10/5(18X,3D18.10/))
 4000 FORMAT(5(30E13.5/))
 5001 FORMAT(5X,'Final Time     :',D20.12)
 5002 FORMAT(5X,'Final Mass     :',D20.12)
 5003 FORMAT(5X,'Polar Angle    :',D20.12)
 5004 FORMAT(5X,'Angle of Mars  :',D20.12)
C -------------------------------------------------------------------
      RETURN
      END




      SUBROUTINE RESET(X,XS,Y,N,M,MS,NMS)
C 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(*),XS(*),Y(NMS,*)
C
      J1=0
      MD=M
      DO 600 J2=1,M
      DO 601 I=1,MS
      IF(XS(I).EQ.X(J2)) GOTO 620
  601 CONTINUE
      J1=J1+1
      X(J1)=X(J2)
      DO 610 I=1,N
  610 Y(I,J1)=Y(I,J2)
      GOTO 600
  620 MD=MD-1
  600 CONTINUE
      M=MD
C -------------------------------------------------------------------
      RETURN
      END
